@@ -48,11 +48,13 @@ from buildlog_consultant.common import (
     MissingRubyFile,
     MissingAutoconfMacro,
     MissingValaPackage,
+    MissingBoostComponents,
     MissingXfceDependency,
     MissingHaskellDependencies,
     MissingVagueDependency,
     DhAddonLoadFailure,
     MissingMavenArtifacts,
+    MissingIntrospectionTypelib,
     GnomeCommonMissing,
     MissingGnomeCommonDependency,
     UnknownCertificateAuthority,
@@ -60,7 +62,12 @@ from buildlog_consultant.common import (
     MissingLibtool,
     MissingQt,
     MissingX11,
+    MissingPerlPredeclared,
+    MissingLatexFile,
+    MissingCargoCrate,
+    MissingStaticLibrary,
 )
+from buildlog_consultant.apt import UnsatisfiedAptDependencies
 
 from .fix_build import BuildFixer
 from .requirements import (
@@ -84,6 +91,7 @@ from .requirements import (
     CMakefileRequirement,
     HaskellPackageRequirement,
     MavenArtifactRequirement,
+    BoostComponentRequirement,
     GnomeCommonRequirement,
     JDKFileRequirement,
     JDKRequirement,
@@ -99,6 +107,11 @@ from .requirements import (
     X11Requirement,
     LibtoolRequirement,
     VagueDependencyRequirement,
+    IntrospectionTypelibRequirement,
+    PerlPreDeclaredRequirement,
+    LatexPackageRequirement,
+    CargoCrateRequirement,
+    StaticLibraryRequirement,
 )
 from .resolver import UnsatisfiedRequirements
 
@@ -112,6 +125,8 @@ def problem_to_upstream_requirement(problem):  # noqa: C901
         return PkgConfigRequirement(problem.module, problem.minimum_version)
     elif isinstance(problem, MissingCHeader):
         return CHeaderRequirement(problem.header)
+    elif isinstance(problem, MissingIntrospectionTypelib):
+        return IntrospectionTypelibRequirement(problem.library)
     elif isinstance(problem, MissingJavaScriptRuntime):
         return JavaScriptRuntimeRequirement()
     elif isinstance(problem, MissingRubyGem):
@@ -120,6 +135,8 @@ def problem_to_upstream_requirement(problem):  # noqa: C901
         return ValaPackageRequirement(problem.package)
     elif isinstance(problem, MissingGoPackage):
         return GoPackageRequirement(problem.package)
+    elif isinstance(problem, MissingBoostComponents):
+        return [BoostComponentRequirement(name) for name in problem.components]
     elif isinstance(problem, DhAddonLoadFailure):
         return DhAddonRequirement(problem.path)
     elif isinstance(problem, MissingPhpClass):
@@ -128,8 +145,14 @@ def problem_to_upstream_requirement(problem):  # noqa: C901
         return RPackageRequirement(problem.package, problem.minimum_version)
     elif isinstance(problem, MissingNodeModule):
         return NodeModuleRequirement(problem.module)
+    elif isinstance(problem, MissingStaticLibrary):
+        return StaticLibraryRequirement(problem.library, problem.filename)
     elif isinstance(problem, MissingNodePackage):
         return NodePackageRequirement(problem.package)
+    elif isinstance(problem, MissingLatexFile):
+        if problem.filename.endswith('.sty'):
+            return LatexPackageRequirement(problem.filename[:-4])
+        return None
     elif isinstance(problem, MissingVagueDependency):
         return VagueDependencyRequirement(problem.name, minimum_version=problem.minimum_version)
     elif isinstance(problem, MissingLibrary):
@@ -169,6 +192,15 @@ def problem_to_upstream_requirement(problem):  # noqa: C901
         return LibtoolRequirement()
     elif isinstance(problem, UnknownCertificateAuthority):
         return CertificateAuthorityRequirement(problem.url)
+    elif isinstance(problem, MissingPerlPredeclared):
+        ret = PerlPreDeclaredRequirement(problem.name)
+        try:
+            return ret.lookup_module()
+        except KeyError:
+            return ret
+    elif isinstance(problem, MissingCargoCrate):
+        # TODO(jelmer): handle problem.requirements
+        return CargoCrateRequirement(problem.crate)
     elif isinstance(problem, MissingSetupPyCommand):
         if problem.command == "test":
             return PythonPackageRequirement("setuptools")
@@ -207,6 +239,9 @@ def problem_to_upstream_requirement(problem):  # noqa: C901
             python_version=problem.python_version,
             minimum_version=problem.minimum_version,
         )
+    elif isinstance(problem, UnsatisfiedAptDependencies):
+        from .resolver.apt import AptRequirement
+        return AptRequirement(problem.relations)
     else:
         return None
 
